@@ -29,6 +29,8 @@ SEXP rzmq_unserialize(SEXP data, SEXP rho);
 
 extern "C" {
   SEXP get_zmq_version();
+  SEXP get_zmq_errno();
+  SEXP get_zmq_strerror();
   SEXP initContext();
   SEXP initSocket(SEXP context_, SEXP socket_type_);
   SEXP bindSocket(SEXP socket_, SEXP address_);
@@ -37,7 +39,7 @@ extern "C" {
   SEXP sendNullMsg(SEXP socket_, SEXP send_more_);
   SEXP receiveNullMsg(SEXP socket_);
   SEXP sendRawString(SEXP socket_, SEXP data_, SEXP send_more_);
-  SEXP receiveSocket(SEXP socket_);
+  SEXP receiveSocket(SEXP socket_, SEXP flags_);
   SEXP receiveString(SEXP socket_);
   SEXP receiveInt(SEXP socket_);
   SEXP receiveDouble(SEXP socket_);
@@ -58,6 +60,9 @@ extern "C" {
   SEXP set_zmq_backlog(SEXP socket_, SEXP option_value_);
   SEXP set_reconnect_ivl_max(SEXP socket_, SEXP option_value_);
   SEXP get_rcvmore(SEXP socket_);
+  SEXP pollSocket(SEXP socket_, SEXP events_, SEXP timeout_);
+  SEXP get_sndtimeo(SEXP socket_);
+  SEXP set_sndtimeo(SEXP socket_, SEXP option_value_);
 }
 
 #endif // INTERFACE_HPP
